#include "main.h"

/****************************************************************************************************************************************
*	Function name:  Init_Interrupts
*	Description	:	This function enables or disable the ADCC interupt.
*					In this examples, MDMA channels are clearing the ADCC frame completion interrupts. So, these interrupts are disabled.
*					Only ADCC Error interrupt and is enabled and that for ADCC trigger input overrun conditions and event miss condition. 
*					other error interrupts such as event collision or memory/address errors are not enabled.
*	Arguments	:	None. 
*   Return value:   None.
****************************************************************************************************************************************/

void Init_Interrupts(void)
{
  	*pREG_ADCC0_ERRMSK_SET = 0x0000007C;		//	1: disable, 0: enable
	*pREG_ADCC0_EIMSK_SET  = 0x00FFFFFF;		//	1: disable, 0: enable
	*pREG_ADCC0_FIMSK_SET  = 0x00000003;		//	1: disable, 0: enable
	
	NVIC_SetPriority(INTR_ADCC0_ERR,0);   		// NVIC_SetPriority(IRQn_Type IRQn, uint32_t priority)
	NVIC_EnableIRQ(INTR_ADCC0_ERR);				// NVIC_EnableIRQ(IRQn_Type IRQn)
}


/****************************************************************************************************************************************
*	Function name:  ADCC0_ERR_Int_Handler
*	Description	:	When ADCC error condition occurs, this function is called. It checks if the interrupt is dur to trigger overrun condition
					or due to event miss condition. 
*					When trigger overrun condition occurs with MDMA_XCNT_CUR register is zero, then this is normal condition because required number 
*					of ADC samples are received. In this case, the below handler clears this condition and returns to main for further calculation.
*					If trigger overrun condition occurs with MDMA_XCNT_CUR register NON-zero. then this is real error condition and program halts in 
*					a while(1) loop. Similarly it halts in while(1) loop for event miss condition.
*	Arguments	:	None. 
*   Return value:   None.
****************************************************************************************************************************************/

unsigned int ADCC_EVT_MISS[24];

void ADCC0_ERR_Int_Handler (void)
{
	unsigned int temp_ERR_STAT = *pREG_ADCC0_ERRSTAT ;

	if(temp_ERR_STAT & BITM_ADCC_ERRSTAT_TRGOV0)			
	{
	 	*pREG_TIMER0_RUN_CLR  =  (BITM_TIMER_RUN_SET_TMR06) ;
		*pREG_ADCC0_ERRSTAT = BITM_ADCC_ERRSTAT_TRGOV0 ;
		*pREG_ADCC0_FISTAT = BITM_ADCC_FISTAT_FINT0;
		if(*pREG_DMA17_XCNT_CUR != 0)
		  while(1);
	}

	if(temp_ERR_STAT & BITM_ADCC_ERRSTAT_TRGOV1)			
	{
		*pREG_TIMER0_RUN_CLR  =  (BITM_TIMER_RUN_SET_TMR07) ;
		*pREG_ADCC0_ERRSTAT = BITM_ADCC_ERRSTAT_TRGOV1 ;
		*pREG_ADCC0_FISTAT = BITM_ADCC_FISTAT_FINT1;
		if(*pREG_DMA19_XCNT_CUR != 0)
		  while(1);
	}
	
	unsigned int temp_EMISS_STAT, EMISS_CNT ;
	if(temp_ERR_STAT & BITM_ADCC_ERRSTAT_EMIS)
	{
		for(temp_EMISS_STAT = *pREG_ADCC0_EMISS, EMISS_CNT = 0; temp_EMISS_STAT != 0; temp_EMISS_STAT = (temp_EMISS_STAT >> 1), 	EMISS_CNT++)
		{
			if(temp_EMISS_STAT & 1)
			{
				*pREG_ADCC0_EMISS = (1 << EMISS_CNT);
				ADCC_EVT_MISS[EMISS_CNT]++;
					while(1);
			}	
		}	
	}
}


/****************************************************************************************************************************************
*	Function name: 	Init_MDMA
*	Description	:	This functions configure the MDMA channels for clearing the ADCC frame completion interrupt latch.
*					The source channel of MDMA is configured in STOP mode for the count equal to required ADCC frames (as defined in BUFF_SIZE macro in main.h) 
*					The destination channel of MDMA is configued in Auto buffer mode with XCOUNT = 1 and in trigger wait mode. Therefore, it waits for a trigger 
*					signal for each transfer. 
*					First MDMA pair (17-18) clears the frame completion interrupt latch bit for ADCC Timer0. Therefore, destination channel of this MDMA accepts 
*					trigger signal from ADCC Timer0. Whenever ADCC TImer0 frame is completd, it gives trigger to MDMA18 channel and then MDMA pair writes ADCC_FISTAT
*					register to clear related interrupt latch.
*					Similarly, second MDMA pair (19-20) clears the frame completion interrupt latch bit for ADCC Timer1. Therefore, destination channel of this MDMA accepts 
*					trigger signal from ADCC Timer1. Whenever ADCC TImer1 frame is completd, it gives trigger to MDMA20 channel and then MDMA pair writes ADCC_FISTAT
*					register to clear related interrupt latch.
*	Arguments	:	None. 
*   Return value:   None.
****************************************************************************************************************************************/

unsigned int ADCC_TMR0_CLR = BITM_ADCC_FISTAT_FINT0, ADCC_TMR1_CLR = BITM_ADCC_FISTAT_FINT1;

void Init_MDMA(void)
{
  *pREG_DMA17_CFG  = 0x00;  *pREG_DMA17_STAT = *pREG_DMA17_STAT;
  *pREG_DMA18_CFG  = 0x00;  *pREG_DMA18_STAT = *pREG_DMA18_STAT;
  *pREG_DMA19_CFG  = 0x00;  *pREG_DMA19_STAT = *pREG_DMA19_STAT;
  *pREG_DMA20_CFG  = 0x00;  *pREG_DMA20_STAT = *pREG_DMA20_STAT;
  
  *pREG_CRC0_CTL = 0x00;  *pREG_CRC0_STAT = *pREG_CRC0_STAT;
  
  	*pREG_TRU0_GCTL  = ((1 << BITP_TRU_GCTL_EN) & BITM_TRU_GCTL_EN);
	*pREG_TRU0_SSR40 = ((TRGM_ADCC0_TMR0_EVT << BITP_TRU_SSR_SSR) & BITM_TRU_SSR_SSR);		//	MDMA0
	*pREG_TRU0_SSR42 = ((TRGM_ADCC0_TMR1_EVT << BITP_TRU_SSR_SSR) & BITM_TRU_SSR_SSR);		//	MDMA1

//	MDMA to clear ADCC Timer0 interrupt
	//	source
	*pREG_DMA17_ADDRSTART = &ADCC_TMR0_CLR;	
	*pREG_DMA17_XCNT      = BUFF_SIZE-1;	
	*pREG_DMA17_XMOD	  = 0;	
	*pREG_DMA17_CFG       = ENUM_DMA_CFG_EN | ENUM_DMA_CFG_MSIZE04;	
	//	Destination
	*pREG_DMA18_ADDRSTART = (void*)REG_ADCC0_FISTAT;//0x4000E024;	
	*pREG_DMA18_XCNT	  = 1;	
	*pREG_DMA18_XMOD	  = 4;	
	*pREG_DMA18_CFG 	  = ENUM_DMA_CFG_EN | ENUM_DMA_CFG_TRGWAIT | ENUM_DMA_CFG_AUTO |ENUM_DMA_CFG_MSIZE04 | ENUM_DMA_CFG_WRITE;

//	MDMA to clear ADCC Timer1 interrupt
	//	source	
	*pREG_DMA19_ADDRSTART = &ADCC_TMR1_CLR;	
	*pREG_DMA19_XCNT      = BUFF_SIZE-1;	
	*pREG_DMA19_XMOD	  = 0;	
	*pREG_DMA19_CFG       = ENUM_DMA_CFG_EN | ENUM_DMA_CFG_MSIZE04;	
	//	Destination
	*pREG_DMA20_ADDRSTART = (void*)REG_ADCC0_FISTAT;//0x4000E024;
	*pREG_DMA20_XCNT	  = 1;	
	*pREG_DMA20_XMOD	  = 4;	
	*pREG_DMA20_CFG 	  = ENUM_DMA_CFG_EN | ENUM_DMA_CFG_TRGWAIT | ENUM_DMA_CFG_AUTO |ENUM_DMA_CFG_MSIZE04 | ENUM_DMA_CFG_WRITE ;	
}
/****************************************************************************************************************************************/
